@@ -1,13 +1,12 @@
 #' @title Extract residual correlations from gllvm object
-#' @description  Calculates the residual covariance and correlation matrices for gllvm model.
+#' @description  Calculates the residual correlation matrix for gllvm model.
 #'
 #' @param object   an object of class 'gllvm'.
-#'
-#' @return Function returns following components:
-#'  \item{cor }{residual correlation matrix}
-#'  \item{cov }{residual covariance matrix}
-#'  \item{trace }{trace of residual covariance matrix}
-#'
+#' @param adjust  The type of adjustment used for  negative binomial and binomial distribution when computing residual correlation matrix. Options are 0 (no adjustment), 1 (the default adjustment) and 2 (alternative adjustment for NB distribution). See details.
+#' 
+#' @details
+#' Residual correlation matrix is calculated based on the residual covariance matrix, see details from \code{\link{getResidualCov.gllvm}}.
+#' 
 #' @author Francis K.C. Hui, Jenni Niku, David I. Warton
 #'
 #' @examples
@@ -18,16 +17,12 @@
 #'fit <- gllvm(y = y, family = poisson())
 #'# residual correlations:
 #'cr <- getResidualCor(fit)
-#'
 #'\dontrun{
 #'# Plot residual correlations:
 #'install.packages("corrplot", "gclus")
 #'library(corrplot)
 #'library(gclus)
-#'rbPal <- colorRampPalette(c('darkblue', 'white', 'darkred'))
-#'breaks <- seq(min(cr$cor), max(cr$cor), length.out = 40)
-#'Colors <- rbPal(100)[as.numeric(cut(cr$cor, breaks = breaks))]
-#'corrplot(cr$cor[order.single(cr$cor), order.single(cr$cor)], diag = F,
+#'corrplot(cr[order.single(cr), order.single(cr)], diag = F,
 #'   type = "lower", method = "square", tl.cex = 0.8, tl.srt = 45, tl.col = "red")
 #'   }
 #'
@@ -35,19 +30,20 @@
 #'@method getResidualCor gllvm
 #'@export
 #'@export getResidualCor.gllvm
-getResidualCor.gllvm = function(object)
+getResidualCor.gllvm = function(object, adjust = 1)
 {
-  ResCov <- object$params$theta %*% t(object$params$theta)
+  ResCov <- getResidualCov.gllvm(object, adjust = adjust)$cov
   Res.sd <- 1 / sqrt(diag(ResCov))
-  Res.Cor <- diag(Res.sd) %*% ResCov %*% diag(Res.sd) * 0.99999
+  Res.Cor <- diag(Res.sd) %*% ResCov %*% diag(Res.sd)
   colnames(Res.Cor) <- colnames(object$y)
   rownames(Res.Cor) <- colnames(object$y)
-  out <- list(cor = Res.Cor, cov = ResCov, trace = sum(diag(ResCov)))
+  Res.Cor[abs(Res.Cor) > 1] <- 1 * sign(Res.Cor[abs(Res.Cor) > 1])
+  out <- Res.Cor
   return(out)
 }
 
 #'@export getResidualCor
-getResidualCor <- function(object)
+getResidualCor <- function(object, adjust)
 {
   UseMethod(generic = "getResidualCor")
 }
